# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF


class ShareGroupsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupsNegativeTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupsNegativeTest, cls).resource_setup()
        # Create a share type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # Create a share group type
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']

        # Create a share group
        cls.share_group_name = data_utils.rand_name("tempest-sg-name")
        cls.share_group_desc = data_utils.rand_name("tempest-sg-description")
        cls.share_group = cls.create_share_group(
            name=cls.share_group_name,
            description=cls.share_group_desc,
            share_group_type_id=cls.share_group_type_id,
            share_type_ids=[cls.share_type_id],
        )
        # Create a share in the share group
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.share_size = CONF.share.share_size
        cls.share = cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            size=cls.share_size,
            share_type_id=cls.share_type_id,
            share_group_id=cls.share_group['id'],
        )
        if CONF.share.run_snapshot_tests:
            # Create a share group snapshot of the share group
            cls.sg_snap_name = data_utils.rand_name("tempest-sg-snap-name")
            cls.sg_snap_desc = data_utils.rand_name(
                "tempest-group-snap-description")
            cls.sg_snapshot = cls.create_share_group_snapshot_wait_for_active(
                cls.share_group['id'],
                name=cls.sg_snap_name,
                description=cls.sg_snap_desc
            )

    @decorators.idempotent_id('7ce3fb52-1bec-42b1-9b4f-671c8465764b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_invalid_source_sg_snapshot_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            source_share_group_snapshot_id='foobar',
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('43c7a454-06b4-4c6e-8aaf-34709db64e28')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_nonexistent_source_sg_snapshot_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            source_share_group_snapshot_id=self.share['id'],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('aae1b1db-ea04-4a53-88ad-e5ee648fe938')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_invalid_share_network_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            share_network_id='foobar',
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('ea817e28-08b9-40c1-bbab-a8820ec564ac')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_group_with_nonexistent_share_network_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            share_network_id=self.share['id'],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('8bc89858-61de-49f3-868c-394841a93503')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_invalid_share_type_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            share_type_ids=['foobar'],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('97183b15-d150-4c6b-b812-734c2500afe7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_nonexistent_share_type_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group,
            share_type_ids=[self.share['id']],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('25a829e2-be7d-4a4d-881e-bc0634515985')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_snapshot_with_invalid_sg_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group_snapshot_wait_for_active,
            'foobar',
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('16ad5a77-0ef7-4906-8e14-56703c2c9d71')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_snapshot_with_nonexistent_sg_id_value_min(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share_group_snapshot_wait_for_active,
            self.share['id'],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('d486a184-1160-4664-ad9b-6f8974685343')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_sg_with_invalid_id_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_group,
            "invalid_share_group_id",
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('7a5dcec1-27cc-4bfe-a166-d8c1832a4dc7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_sg_without_passing_group_id_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_group,
            '', version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('9127933a-1960-415e-bf21-8bcb5720e371')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_sg_with_invalid_id_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.update_share_group,
            'invalid_share_group_id',
            name='new_name',
            description='new_description',
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('bf8f9865-a906-4d55-b233-fec57cf43b66')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_sg_with_invalid_id_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.delete_share_group,
            "invalid_share_group_id",
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('16431501-800f-4695-bae6-6a4c715c2fbf')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_sg_without_passing_sg_id_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.delete_share_group,
            '', version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('18fe2dee-4a07-484e-8f0f-bbc238500dc3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_delete_sg_in_use_by_sg_snapshot_min(self):
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_v2_client.delete_share_group,
            self.share_group['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('d2a58f10-cc86-498d-a5e0-1468d4345852')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_delete_share_in_use_by_sg_snapshot_min(self):
        params = {'share_group_id': self.share['share_group_id']}
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.delete_share,
            self.share['id'],
            params=params,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('c2225b19-d5f5-4d15-bb9a-de63bfce6760')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_sg_containing_a_share_min(self):
        self.assertRaises(
            lib_exc.Conflict,
            self.shares_v2_client.delete_share_group,
            self.share_group['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        # Verify share group is not put into error state from conflict
        sg = self.shares_v2_client.get_share_group(
            self.share_group['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)['share_group']
        self.assertEqual('available', sg['status'])

    @decorators.idempotent_id('edd329b8-7188-481f-9445-8f6d913538fa')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_filter_shares_on_invalid_group_id_min(self):
        shares = self.shares_v2_client.list_shares(
            detailed=True,
            params={'share_group_id': 'foobar'},
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )['shares']
        self.assertEqual(0, len(shares), 'Incorrect number of shares returned')

    @decorators.idempotent_id('5dc10968-cbff-46d9-a1aa-bafccc7a1905')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_filter_shares_on_nonexistent_group_id_min(self):
        shares = self.shares_v2_client.list_shares(
            detailed=True,
            params={'share_group_id': self.share['id']},
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )['shares']
        self.assertEqual(0, len(shares), 'Incorrect number of shares returned')

    @decorators.idempotent_id('f805f683-fe05-4534-9f40-a74be42ff82b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_filter_shares_on_empty_share_group_id_min(self):
        share_group = self.create_share_group(
            name='tempest_sg',
            description='tempest_sg_desc',
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )
        shares = self.shares_v2_client.list_shares(
            detailed=True,
            params={'share_group_id': share_group['id']},
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )['shares']
        self.assertEqual(0, len(shares), 'Incorrect number of shares returned')

    @decorators.idempotent_id('8fc20c22-082f-4851-bcc3-d2f3af57f027')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_with_nonexistent_az_min(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.create_share_group,
            name='tempest_sg',
            description='tempest_sg_desc',
            availability_zone='fake_nonexistent_az',
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @utils.skip_if_microversion_not_supported("2.34")
    @decorators.idempotent_id('64527564-9cd6-42db-8897-910f4fc1a151')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_sg_and_share_with_different_azs(self):
        azs = self.shares_v2_client.list_availability_zones(
            )['availability_zones']

        if len(azs) < 2:
            raise self.skipException(
                'Test requires presence of at least 2 availability zones.')
        else:
            share_group = self.shares_v2_client.get_share_group(
                self.share_group['id'], '2.34')['share_group']
            different_az = [
                az['name']
                for az in azs
                if az['name'] != share_group['availability_zone']
            ][0]

        self.assertRaises(
            lib_exc.BadRequest,
            self.create_share,
            share_type_id=self.share_type_id,
            share_group_id=self.share_group['id'],
            availability_zone=different_az,
            version='2.34')



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareNetworkListMixin(object):

    @decorators.idempotent_id('41c635b1-d9ef-4c05-9100-5e4b0034b523')
    @tc.attr("gate", "smoke", )
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks(self):
        listed = self.shares_client.list_share_networks()['share_networks']
        any(self.sn_with_ldap_ss["id"] in sn["id"] for sn in listed)

        # verify keys
        keys = ["name", "id"]
        [self.assertIn(key, sn.keys()) for sn in listed for key in keys]

    @decorators.idempotent_id('18fe9031-cefc-4df3-bbb0-6541f5fda12b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_try_list_share_networks_all_tenants(self):
        listed = self.shares_client.list_share_networks_with_detail(
            params={'all_tenants': 1})['share_networks']
        any(self.sn_with_ldap_ss["id"] in sn["id"] for sn in listed)

        # verify keys
        keys = ["name", "id"]
        [self.assertIn(key, sn.keys()) for sn in listed for key in keys]

    @decorators.idempotent_id('caf6635a-ecb8-4981-9776-11dcfdf3cdbc')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_try_list_share_networks_project_id(self):
        listed = self.shares_client.list_share_networks_with_detail(
            params={'project_id': 'some_project'})['share_networks']
        any(self.sn_with_ldap_ss["id"] in sn["id"] for sn in listed)

        # verify keys
        keys = ["name", "id"]
        [self.assertIn(key, sn.keys()) for sn in listed for key in keys]

    @decorators.idempotent_id('285c7a91-1703-42a5-86c8-2463edde60e2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks_with_detail(self):
        listed = self.shares_v2_client.list_share_networks_with_detail(
            )['share_networks']
        any(self.sn_with_ldap_ss["id"] in sn["id"] for sn in listed)

        # verify keys
        keys = [
            "name", "id", "description", "network_type",
            "project_id", "cidr", "ip_version",
            "neutron_net_id", "neutron_subnet_id",
            "created_at", "updated_at", "segmentation_id",
        ]

        # In v2.18 and beyond, we expect gateway.
        if utils.is_microversion_supported('2.18'):
            keys.append('gateway')

        # In v2.20 and beyond, we expect mtu.
        if utils.is_microversion_supported('2.20'):
            keys.append('mtu')

        # In v2.51 and beyond, share-network does not have
        # network parameters anymore.
        if utils.is_microversion_supported('2.51'):
            subnet_keys = [
                "network_type", "cidr", "ip_version", "neutron_net_id",
                "neutron_subnet_id", "segmentation_id", "gateway", "mtu"
            ]
            keys = list(set(keys) - set(subnet_keys))
            keys.append('share_network_subnets')
            for sn in listed:
                [self.assertIn(key, list(subnet.keys())) for key in subnet_keys
                 for subnet in sn['share_network_subnets']]

        [self.assertIn(key, sn.keys()) for sn in listed for key in keys]

    @decorators.idempotent_id('6d13a090-0855-40c0-85b1-424f6878c6ce')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks_filter_by_ss(self):
        listed = self.shares_client.list_share_networks_with_detail(
            {'security_service_id': self.ss_ldap['id']})['share_networks']
        self.assertTrue(any(self.sn_with_ldap_ss['id'] == sn['id']
                            for sn in listed))
        for sn in listed:
            ss_list = self.shares_client.list_sec_services_for_share_network(
                sn['id'])['security_services']
            self.assertTrue(any(ss['id'] == self.ss_ldap['id']
                                for ss in ss_list))

    @decorators.idempotent_id('bff1356e-70aa-4bbe-b398-cb4dadd8fcb1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_share_networks_like_filter(self):
        valid_filter_opts = {
            'name': 'sn_with_ldap_ss',
            'description': 'fake',
        }

        listed = self.shares_v2_client.list_share_networks_with_detail(
            {'name~': 'ldap_ss', 'description~': 'fa'})['share_networks']
        self.assertTrue(any(self.sn_with_ldap_ss['id'] == sn['id']
                            for sn in listed))
        for sn in listed:
            self.assertTrue(all(value in sn[key] for key, value in
                                valid_filter_opts.items()))

    @decorators.idempotent_id('27490442-60f8-4514-a10a-194a400b48bb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_share_networks_all_filter_opts(self):
        valid_filter_opts = {
            'created_before': '2002-10-10',
            'created_since': '2001-01-01',
            'neutron_net_id': '1111',
            'neutron_subnet_id': '2222',
            'network_type': 'vlan',
            'segmentation_id': 1000,
            'cidr': '10.0.0.0/24',
            'ip_version': 4,
            'name': 'sn_with_ldap_ss'
        }

        listed = self.shares_client.list_share_networks_with_detail(
            valid_filter_opts)['share_networks']
        self.assertTrue(any(self.sn_with_ldap_ss['id'] == sn['id']
                            for sn in listed))
        created_before = valid_filter_opts.pop('created_before')
        created_since = valid_filter_opts.pop('created_since')
        for sn in listed:
            self.assertTrue(all(sn[key] == value for key, value in
                                valid_filter_opts.items()))
            self.assertLessEqual(sn['created_at'], created_before)
            self.assertGreaterEqual(sn['created_at'], created_since)


class ShareNetworksTest(base.BaseSharesMixedTest, ShareNetworkListMixin):

    @classmethod
    def resource_setup(cls):
        super(ShareNetworksTest, cls).resource_setup()

        # create share_type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        ss_data = utils.generate_security_service_data()
        cls.ss_ldap = cls.create_security_service(**ss_data)

        cls.data_sn_with_ldap_ss = {
            'name': 'sn_with_ldap_ss',
            'neutron_net_id': '1111',
            'neutron_subnet_id': '2222',
            'created_at': '2002-02-02',
            'updated_at': None,
            'network_type': 'vlan',
            'segmentation_id': 1000,
            'cidr': '10.0.0.0/24',
            'ip_version': 4,
            'description': 'fake description',
        }
        cls.sn_with_ldap_ss = cls.create_share_network(
            cleanup_in_class=True,
            add_security_services=False,
            **cls.data_sn_with_ldap_ss)

        cls.shares_client.add_sec_service_to_share_network(
            cls.sn_with_ldap_ss["id"],
            cls.ss_ldap["id"])

        cls.data_sn_with_kerberos_ss = {
            'name': 'sn_with_kerberos_ss',
            'created_at': '2003-03-03',
            'updated_at': None,
            'neutron_net_id': 'test net id',
            'neutron_subnet_id': 'test subnet id',
            'network_type': 'local',
            'segmentation_id': 2000,
            'cidr': '10.0.0.0/13',
            'ip_version': 6,
            'description': 'fake description',
        }

        cls.ss_kerberos = cls.create_security_service(
            ss_type='kerberos',
            **cls.data_sn_with_ldap_ss)

        cls.sn_with_kerberos_ss = cls.create_share_network(
            cleanup_in_class=True,
            add_security_services=False,
            **cls.data_sn_with_kerberos_ss)

        cls.shares_client.add_sec_service_to_share_network(
            cls.sn_with_kerberos_ss["id"],
            cls.ss_kerberos["id"])

    @decorators.idempotent_id('b998a594-f630-475d-b46f-e4143caf61fb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_delete_share_network(self):
        # generate data for share network
        data = utils.generate_share_network_data()

        # create share network
        created = self.shares_client.create_share_network(
            **data)['share_network']
        self.assertLessEqual(data.items(), created.items())

        # Delete share_network
        self.shares_client.delete_share_network(created["id"])

    @decorators.idempotent_id('55990ec2-37f0-483f-9c67-76fd6f377cc1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_network(self):
        get = self.shares_client.get_share_network(
            self.sn_with_ldap_ss["id"])['share_network']
        self.assertEqual('2002-02-02T00:00:00.000000', get['created_at'])
        data = self.data_sn_with_ldap_ss.copy()
        del data['created_at']
        self.assertLessEqual(data.items(), get.items())

    @decorators.idempotent_id('1837fdd3-8068-4e88-bc50-9224498f84c0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_share_network(self):
        update_data = utils.generate_share_network_data()
        updated = self.shares_client.update_share_network(
            self.sn_with_ldap_ss["id"],
            **update_data)['share_network']
        self.assertLessEqual(update_data.items(), updated.items())

    @decorators.idempotent_id('198a5c08-3aaf-4623-9720-95d33ebe3376')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_update_valid_keys_sh_server_exists(self):
        self.create_share(share_type_id=self.share_type_id,
                          cleanup_in_class=False)
        update_dict = {
            "name": "new_name",
            "description": "new_description",
        }
        updated = self.shares_client.update_share_network(
            self.shares_client.share_network_id,
            **update_dict)['share_network']
        self.assertLessEqual(update_dict.items(), updated.items())

    @decorators.idempotent_id('7595a844-a28e-476c-89f1-4d3193ce9d5b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_recreate_share_network(self):
        # generate data for share network
        data = utils.generate_share_network_data()

        # create share network
        sn1 = self.shares_client.create_share_network(**data)['share_network']
        self.assertLessEqual(data.items(), sn1.items())

        # Delete first share network
        self.shares_client.delete_share_network(sn1["id"])

        # create second share network with same data
        sn2 = self.shares_client.create_share_network(**data)['share_network']
        self.assertLessEqual(data.items(), sn2.items())

        # Delete second share network
        self.shares_client.delete_share_network(sn2["id"])

    @decorators.idempotent_id('be5f4f60-493e-47ea-a5bd-f16dfaa98c5c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_two_share_networks_with_same_net_and_subnet(self):
        # generate data for share network
        data = utils.generate_share_network_data()

        # create first share network
        sn1 = self.create_share_network(**data)
        self.assertLessEqual(data.items(), sn1.items())

        # create second share network
        sn2 = self.create_share_network(**data)
        self.assertLessEqual(data.items(), sn2.items())

    @decorators.idempotent_id('2dbf91da-04ae-4f9f-a7b9-0299c6b2e02c')
    @testtools.skipUnless(CONF.share.create_networks_when_multitenancy_enabled,
                          "Only for setups with network creation.")
    @testtools.skipUnless(CONF.share.multitenancy_enabled,
                          "Only for multitenancy.")
    @testtools.skipUnless(CONF.service_available.neutron, "Only with neutron.")
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_gateway_mtu_neutron_net_id_with_neutron(self):
        self.create_share(share_type_id=self.share_type_id,
                          cleanup_in_class=False)
        share_net_details = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id)['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(share_net_details)
            if utils.share_network_subnets_are_supported()
            else share_net_details)

        if utils.is_microversion_supported('2.18'):
            subnet_details = self.subnets_client.show_subnet(
                share_net_info['neutron_subnet_id'])
            self.assertEqual(subnet_details['subnet']['gateway_ip'],
                             share_net_info['gateway'])

        if utils.is_microversion_supported('2.20'):
            network_details = self.networks_client.show_network(
                share_net_info['neutron_net_id'])
            self.assertEqual(network_details['network']['mtu'],
                             share_net_info['mtu'])
            self.assertEqual(network_details['network']['id'],
                             share_net_info['neutron_net_id'])



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from oslo_utils import uuidutils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


class ScenarioTest(manager.NetworkScenarioTest):
    """Base class for scenario tests. Uses tempest own clients. """

    # ## Test functions library
    #
    # The create_[resource] functions only return body and discard the
    # resp part which is not used in scenario tests

    def _image_create(self, name, fmt, path,
                      disk_format=None, properties=None):
        if properties is None:
            properties = {}
        name = data_utils.rand_name('%s-' % name)
        params = {
            'name': name,
            'container_format': fmt,
            'disk_format': disk_format or fmt,
            'visibility': 'private'
        }
        # Additional properties are flattened out in the v2 API.
        params.update(properties)
        body = self.image_client.create_image(**params)
        image = body['image'] if 'image' in body else body
        self.addCleanup(self.image_client.delete_image, image['id'])
        self.assertEqual("queued", image['status'])
        with open(path, 'rb') as image_file:
            self.image_client.store_image_file(image['id'], image_file)
        return image['id']

    def glance_image_create(self):
        img_path = CONF.scenario.img_file
        img_container_format = CONF.scenario.img_container_format
        img_disk_format = CONF.scenario.img_disk_format
        img_properties = CONF.scenario.img_properties
        LOG.debug("paths: img: %s, container_format: %s, disk_format: %s, "
                  "properties: %s",
                  img_path, img_container_format, img_disk_format,
                  img_properties)
        image = self._image_create('scenario-img',
                                   img_container_format,
                                   img_path,
                                   disk_format=img_disk_format,
                                   properties=img_properties)
        LOG.debug("image:%s", image)

        return image

    def _log_net_info(self, exc):
        # network debug is called as part of ssh init
        if not isinstance(exc, lib_exc.SSHTimeout):
            LOG.debug('Network information on a devstack host')


class NetworkScenarioTest(ScenarioTest):
    """Base class for network scenario tests.

    This class provide helpers for network scenario tests, using the neutron
    API. Helpers from ancestor which use the nova network API are overridden
    with the neutron API.

    This Class also enforces using Neutron instead of novanetwork.
    Subclassed tests will be skipped if Neutron is not enabled

    """

    @classmethod
    def skip_checks(cls):
        super(NetworkScenarioTest, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException('Neutron not available')

    def _get_network_by_name_or_id(self, identifier):

        if uuidutils.is_uuid_like(identifier):
            return self.os_admin.networks_client.show_network(
                identifier)['network']

        networks = self.os_admin.networks_client.list_networks(
            name=identifier)['networks']
        self.assertNotEqual(len(networks), 0,
                            "Unable to get network by name: %s" % identifier)
        return networks[0]

    def create_floating_ip(self, thing, external_network_id=None,
                           port_id=None, ip_addr=None, client=None):
        """Create a floating IP and associates to a resource/port on Neutron"""
        if not external_network_id:
            external_network_id = CONF.network.public_network_id
        if not client:
            client = self.floating_ips_client
        if not port_id:
            port_id, ip4 = self.get_server_port_id_and_ip4(thing,
                                                           ip_addr=ip_addr)
        else:
            ip4 = None
        result = client.create_floatingip(
            floating_network_id=external_network_id,
            port_id=port_id,
            tenant_id=thing['tenant_id'],
            fixed_ip_address=ip4
        )
        floating_ip = result['floatingip']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        client.delete_floatingip,
                        floating_ip['id'])
        return floating_ip

    def create_loginable_secgroup_rule(self, security_group_rules_client=None,
                                       secgroup=None,
                                       security_groups_client=None):
        """Create loginable security group rule

        This function will create:
        1. egress and ingress tcp port 22 allow rule in order to allow ssh
        access for ipv4.
        2. egress and ingress ipv6 icmp allow rule, in order to allow icmpv6.
        3. egress and ingress ipv4 icmp allow rule, in order to allow icmpv4.
        """

        if security_group_rules_client is None:
            security_group_rules_client = self.security_group_rules_client
        if security_groups_client is None:
            security_groups_client = self.security_groups_client
        rules = []
        rulesets = [
            dict(
                # ssh
                protocol='tcp',
                port_range_min=22,
                port_range_max=22,
            ),
            dict(
                # ipv6-ssh
                protocol='tcp',
                port_range_min=22,
                port_range_max=22,
                ethertype='IPv6',
            ),
            dict(
                # ping
                protocol='icmp',
            ),
            dict(
                # ipv6-icmp for ping6
                protocol='icmp',
                ethertype='IPv6',
            )
        ]
        sec_group_rules_client = security_group_rules_client
        for ruleset in rulesets:
            for r_direction in ['ingress', 'egress']:
                ruleset['direction'] = r_direction
                try:
                    sg_rule = self.create_security_group_rule(
                        sec_group_rules_client=sec_group_rules_client,
                        secgroup=secgroup,
                        security_groups_client=security_groups_client,
                        **ruleset)
                except lib_exc.Conflict as ex:
                    # if rule already exist - skip rule and continue
                    msg = 'Security group rule already exists'
                    if msg not in ex._error_string:
                        raise ex
                else:
                    self.assertEqual(r_direction, sg_rule['direction'])
                    rules.append(sg_rule)

        return rules



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


@ddt.ddt
class ShareTypesAdminNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ShareTypesAdminNegativeTest, cls).resource_setup()
        cls.extra_specs = cls.add_extra_specs_to_dict({"key": "value"})

    @decorators.idempotent_id('0efe4ed6-9318-4174-aef7-fca4b6aa6444')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_nonexistent_share_type(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_shares_v2_client.create_share,
                          share_type_id=data_utils.rand_name("fake"))

    @decorators.idempotent_id('a1cd6c4f-4dc4-4f45-813b-f1cd3527c614')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type_with_empty_name(self):
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_type, '')

    @decorators.idempotent_id('ca59430b-d1fb-4e8f-b1e3-6ab6a6b40984')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type_with_too_big_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_type,
                          "x" * 256,
                          client=self.admin_shares_v2_client)

    @decorators.idempotent_id('42f7777a-7ee4-4622-80f8-8726b467c4db')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data('2.0', '2.6', '2.40')
    def test_create_share_type_with_description_in_wrong_version(
            self, version):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_share_type,
                          data_utils.rand_name("tempest_type_name"),
                          extra_specs=self.add_extra_specs_to_dict(),
                          description="tempest_type_description",
                          version=version,
                          client=self.admin_shares_v2_client)

    @decorators.idempotent_id('43b0ef5a-3a05-4f74-a08e-57efebb7e66e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_type_by_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_shares_v2_client.get_share_type,
                          data_utils.rand_name("fake"))

    @decorators.idempotent_id('27e00ad7-edc8-4d50-b53e-4da41dd8a5d3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_share_type_by_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_shares_v2_client.delete_share_type,
                          data_utils.rand_name("fake"))

    @decorators.idempotent_id('1f481bab-5205-49ee-bf01-b1848a32f9ee')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_duplicate_of_share_type(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(lib_exc.Conflict,
                          self.create_share_type,
                          st["name"],
                          extra_specs=self.add_extra_specs_to_dict(),
                          client=self.admin_shares_v2_client)

    @decorators.idempotent_id('c13f54eb-17a4-4403-be87-f6a3ca18de6e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_type_allowed_for_public(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(lib_exc.Conflict,
                          self.admin_shares_v2_client.add_access_to_share_type,
                          st["id"],
                          self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('bf1d68fb-b954-4b3b-af54-115f3b67b3b3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_share_type_allowed_for_public(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_shares_v2_client.remove_access_from_share_type,
            st["id"],
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('3d9a7e9d-2b64-422e-b8b6-88c3088564f6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_type_by_nonexistent_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.admin_shares_v2_client.add_access_to_share_type,
                          data_utils.rand_name("fake"),
                          self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('13d5c3cd-15cd-4c2e-ace6-46889cf9e0e2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_remove_share_type_by_nonexistent_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.remove_access_from_share_type,
            data_utils.rand_name("fake"),
            self.admin_shares_v2_client.tenant_id)

    @decorators.idempotent_id('0fd53c51-e1ba-4392-9f4c-5d3bdd157163')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_BACKEND)
    def test_create_share_with_non_allowed_share_type(self):
        # Create a private share type
        name = data_utils.rand_name('share-type')
        share_type = self.create_share_type(
            client=self.admin_shares_v2_client,
            name=name, is_public=False,
            extra_specs=self.add_extra_specs_to_dict())

        # The share type should not be listed without access
        share_type_list = (
            self.admin_shares_v2_client.list_share_types()['share_types'])
        self.assertFalse(
            any(share_type['id'] in st['id'] for st in share_type_list))

        # List projects that have access for share type - none expected
        access = self.admin_shares_v2_client.list_access_to_share_type(
            share_type['id'])['share_type_access']
        self.assertEmpty(access)

        # Although the share type should not be found on alt project,
        # try to create a share with it by using the share type name
        self.assertRaises(lib_exc.NotFound,
                          self.alt_shares_v2_client.create_share,
                          share_type_id=share_type['name'])

        # The share should not be listed
        share_list = self.alt_shares_v2_client.list_shares(
            detailed=True)['shares']
        self.assertFalse(
            any(share_type['id'] in s['share_type'] for s in share_list))



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion
RESTRICTED_RULES_VERSION = '2.82'


def _create_delete_ro_access_rule(self, version):
    """Common test case for usage in test suites with different decorators.

    :param self: instance of test class
    """

    if utils.is_microversion_le(version, '2.9'):
        client = self.shares_client
    else:
        client = self.shares_v2_client

    rule = self.allow_access(
        self.share["id"], client=client, access_type=self.access_type,
        access_to=self.access_to, access_level='ro', version=version)

    self.assertEqual('ro', rule['access_level'])
    for key in ('deleted', 'deleted_at', 'instance_mappings'):
        self.assertNotIn(key, rule.keys())

    # rules must start out in 'new' until 2.28 & 'queued_to_apply' after 2.28
    if utils.is_microversion_le(version, "2.27"):
        self.assertEqual("new", rule['state'])
    else:
        self.assertEqual("queued_to_apply", rule['state'])

        # If the 'access_rules_status' transitions to 'active',
        # rule state must too
        rules = self.shares_v2_client.list_access_rules(
            self.share['id'])['access_list']
        rule = [r for r in rules if r['id'] == rule['id']][0]
        self.assertEqual("active", rule['state'])
    return rule


@ddt.ddt
class ShareIpRulesForNFSTest(base.BaseSharesMixedTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(ShareIpRulesForNFSTest, cls).skip_checks()
        if (cls.protocol not in CONF.share.enable_protocols or
                cls.protocol not in CONF.share.enable_ip_rules_for_protocols):
            msg = "IP rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(ShareIpRulesForNFSTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)
        cls.access_type = "ip"
        cls.access_to = "2.2.2.2"

    @decorators.idempotent_id('3390df2d-f6f8-4634-a562-87c1be994f6a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*itertools.chain(
        itertools.product(
            utils.deduplicate(['1.0', '2.9', '2.37', LATEST_MICROVERSION]),
            [4]),
        itertools.product(
            utils.deduplicate(['2.38', LATEST_MICROVERSION]), [6])
    ))
    @ddt.unpack
    def test_create_delete_access_rules_with_one_ip(self, version,
                                                    ip_version):

        if ip_version == 4:
            access_to = utils.rand_ip()
        else:
            access_to = utils.rand_ipv6_ip()

        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client

        # create rule
        rule = self.allow_access(
            self.share["id"], client=client, access_type=self.access_type,
            access_to=access_to, version=version)

        self.assertEqual('rw', rule['access_level'])
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])

    @decorators.idempotent_id('5d25168a-d646-443e-8cf1-3151eb7887f5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*itertools.chain(
        itertools.product(
            utils.deduplicate(['1.0', '2.9', '2.37', LATEST_MICROVERSION]),
            [4]),
        itertools.product(
            utils.deduplicate(['2.38', LATEST_MICROVERSION]), [6])
    ))
    @ddt.unpack
    def test_create_delete_access_rule_with_cidr(self, version, ip_version):
        if ip_version == 4:
            access_to = utils.rand_ip(network=True)
        else:
            access_to = utils.rand_ipv6_ip(network=True)
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client
        # create rule
        rule = self.allow_access(
            self.share["id"], client=client, access_type=self.access_type,
            access_to=access_to, version=version)

        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())
        self.assertEqual('rw', rule['access_level'])

    @decorators.idempotent_id('187a4fb0-ba1d-45b9-83c9-f0272e7e6f3e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        "nfs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for NFS protocol.")
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_ro_access_rule(self, version):
        _create_delete_ro_access_rule(self, version)

    @utils.skip_if_microversion_not_supported('2.88')
    @decorators.idempotent_id('01940881-6f95-77f8-b47d-0941c4e6bafb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        "nfs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for NFS protocol.")
    def test_update_access_rule(self):
        rule = _create_delete_ro_access_rule(self, LATEST_MICROVERSION)
        rule = self.shares_v2_client.update_access_rule(
            access_id=rule['id'], access_level='rw')['access']
        waiters.wait_for_resource_status(
            self.shares_v2_client, self.share['id'], status='active',
            resource_name='access_rule', rule_id=rule['id'])
        self.assertEqual(rule['access_level'], 'rw')


@ddt.ddt
class ShareIpRulesForCIFSTest(ShareIpRulesForNFSTest):
    protocol = "cifs"

    @decorators.idempotent_id('8fa0a15f-c04c-4521-91e7-020943bede8a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        "cifs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for CIFS protocol.")
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_ro_access_rule(self, version):
        _create_delete_ro_access_rule(self, version)

    @utils.skip_if_microversion_not_supported('2.88')
    @decorators.idempotent_id('02940881-6f95-77f8-b47d-0941c4e6bafb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        "cifs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for CIFS protocol.")
    def test_update_access_rule(self):
        super(
            ShareIpRulesForCIFSTest,
            self
        ).test_update_access_rule()


@ddt.ddt
class ShareUserRulesForNFSTest(base.BaseSharesMixedTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(ShareUserRulesForNFSTest, cls).skip_checks()
        if (cls.protocol not in CONF.share.enable_protocols or
                cls.protocol not in
                CONF.share.enable_user_rules_for_protocols):
            msg = "USER rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(ShareUserRulesForNFSTest, cls).resource_setup()

        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)

        cls.access_type = "user"
        cls.access_to = CONF.share.username_for_user_rules

    @decorators.idempotent_id('1f87565f-c3d9-448d-b89a-387d6c2fdae6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_user_rule(self, version):
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client

        # create rule
        rule = self.allow_access(
            self.share["id"], client=client, access_type=self.access_type,
            access_to=self.access_to, version=version)

        self.assertEqual('rw', rule['access_level'])
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])

    @decorators.idempotent_id('ccb08342-b7ef-4dda-84ba-8de9879d8862')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        "nfs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for NFS protocol.")
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_ro_access_rule(self, version):
        _create_delete_ro_access_rule(self, version)


@ddt.ddt
class ShareUserRulesForCIFSTest(ShareUserRulesForNFSTest):
    protocol = "cifs"

    @decorators.idempotent_id('ee11084d-6c1d-4856-8044-9aa9e6c670fb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        "cifs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for CIFS protocol.")
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_ro_access_rule(self, version):
        _create_delete_ro_access_rule(self, version)


@ddt.ddt
class ShareCertRulesForGLUSTERFSTest(base.BaseSharesMixedTest):
    protocol = "glusterfs"

    @classmethod
    def skip_checks(cls):
        super(ShareCertRulesForGLUSTERFSTest, cls).skip_checks()
        if (cls.protocol not in CONF.share.enable_protocols or
                cls.protocol not in
                CONF.share.enable_cert_rules_for_protocols):
            msg = "Cert rule tests for %s protocol are disabled" % cls.protocol
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(ShareCertRulesForGLUSTERFSTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)

        cls.access_type = "cert"
        # Provide access to a client identified by a common name (CN) of the
        # certificate that it possesses.
        cls.access_to = "client1.com"

    @decorators.idempotent_id('775ebc55-4a4d-4012-a030-2eeb7b6d2ce8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_cert_rule(self, version):
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client

        # create rule
        rule = self.allow_access(
            self.share["id"], client=client, access_type=self.access_type,
            access_to=self.access_to, version=version)

        self.assertEqual('rw', rule['access_level'])
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])

    @decorators.idempotent_id('cdd93d8e-7255-4ed4-8ef0-929a62bb302c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipIf(
        "glusterfs" not in CONF.share.enable_ro_access_level_for_protocols,
        "RO access rule tests are disabled for GLUSTERFS protocol.")
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_create_delete_cert_ro_access_rule(self, version):
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client
        rule = self.allow_access(
            self.share["id"], client=client, access_type='cert',
            access_to='client2.com', access_level='ro', version=version)

        self.assertEqual('ro', rule['access_level'])
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])


@ddt.ddt
class ShareCephxRulesForCephFSTest(base.BaseSharesMixedTest):
    protocol = "cephfs"

    @classmethod
    def skip_checks(cls):
        super(ShareCephxRulesForCephFSTest, cls).skip_checks()
        if (cls.protocol not in CONF.share.enable_protocols or
                cls.protocol not in
                CONF.share.enable_cephx_rules_for_protocols):
            msg = ("Cephx rule tests for %s protocol are disabled." %
                   cls.protocol)
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(ShareCephxRulesForCephFSTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(cls.protocol,
                                     share_type_id=cls.share_type_id)

        cls.access_type = "cephx"
        # Provide access to a client identified by a cephx auth id.
        cls.access_to = "bob"

    @decorators.idempotent_id('4e636fd2-26ef-4b63-96eb-77860a8b6cdf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(*itertools.product(
        utils.deduplicate(['2.13', '2.27', '2.28', LATEST_MICROVERSION]),
        ("alice", "alice_bob", "alice bob"),
        ('rw', 'ro')))
    @ddt.unpack
    def test_create_delete_cephx_rule(self, version, access_to, access_level):
        rule = self.allow_access(
            self.share["id"], access_type=self.access_type,
            access_to=access_to, version=version, access_level=access_level)

        self.assertEqual(access_level, rule['access_level'])
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            self.assertNotIn(key, rule.keys())

    @decorators.idempotent_id('ad907303-a439-4fcb-8845-fe91ecab7dc2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_different_users_in_same_tenant_can_use_same_cephx_id(self):
        # Grant access to the share
        self.allow_access(
            self.share['id'], access_type=self.access_type,
            access_to=self.access_to, access_level='rw')

        # Create a new user in the current project
        project = self.os_admin.projects_client.show_project(
            self.shares_v2_client.tenant_id)['project']
        user_client = self.create_user_and_get_client(project)

        # Create second share by the new user
        share2 = self.create_share(client=user_client.shares_v2_client,
                                   share_protocol=self.protocol,
                                   share_type_id=self.share_type_id)

        # Grant access to the second share using the same cephx ID that was
        # used in access1
        self.allow_access(
            share2['id'], client=user_client.shares_v2_client,
            access_type=self.access_type, access_to=self.access_to,
            access_level='rw')


@ddt.ddt
class ShareRulesTest(base.BaseSharesMixedTest):
    """A Test class to test access rules generically.

    Tests in this class don't care about the type of access rule or the
    protocol of the share created. They are meant to test the API semantics
    of the access rules APIs.
    """

    @classmethod
    def skip_checks(cls):
        super(ShareRulesTest, cls).skip_checks()
        if not (any(p in CONF.share.enable_ip_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_user_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cert_rules_for_protocols
                    for p in cls.protocols) or
                any(p in CONF.share.enable_cephx_rules_for_protocols
                    for p in cls.protocols)):
            cls.message = "Rule tests are disabled"
            raise cls.skipException(cls.message)

    @classmethod
    def resource_setup(cls):
        super(ShareRulesTest, cls).resource_setup()
        cls.access_type, cls.access_to = (
            utils.get_access_rule_data_from_config(
                cls.shares_v2_client.share_protocol)
        )
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        cls.share = cls.create_share(share_type_id=cls.share_type_id)
        cls.user_project = cls.os_admin.projects_client.show_project(
            cls.shares_v2_client.project_id)['project']
        cls.new_user = cls.create_user_and_get_client(
            project=cls.user_project)

    @decorators.idempotent_id('c52e95cc-d6ea-4d02-9b52-cd7c1913dfff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(*utils.deduplicate(
        ['1.0', '2.9', '2.27', '2.28', '2.45', LATEST_MICROVERSION]))
    def test_list_access_rules(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        if (utils.is_microversion_lt(version, '2.13') and
                CONF.share.enable_cephx_rules_for_protocols):
            msg = ("API version %s does not support cephx access type, need "
                   "version >= 2.13." % version)
            raise self.skipException(msg)

        metadata = None
        if utils.is_microversion_ge(version, '2.45'):
            metadata = {'key1': 'v1', 'key2': 'v2'}
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client
        # create rule
        rule = self.allow_access(
            self.share["id"], client=client, access_type=self.access_type,
            access_to=self.access_to, metadata=metadata, version=version)

        # verify added rule keys since 2.33 when create rule
        if utils.is_microversion_ge(version, '2.33'):
            self.assertIn('created_at', list(rule.keys()))
            self.assertIn('updated_at', list(rule.keys()))
        else:
            self.assertNotIn('created_at', list(rule.keys()))
            self.assertNotIn('updated_at', list(rule.keys()))

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])

        # list rules
        if utils.is_microversion_eq(version, '1.0'):
            rules = self.shares_client.list_access_rules(
                self.share["id"])['access_list']
        else:
            rules = self.shares_v2_client.list_access_rules(
                self.share["id"], version=version)['access_list']

        # verify keys
        keys = ("id", "access_type", "access_to", "access_level")
        if utils.is_microversion_ge(version, '2.21'):
            keys += ("access_key", )
        if utils.is_microversion_ge(version, '2.33'):
            keys += ("created_at", "updated_at", )
        if utils.is_microversion_ge(version, '2.45'):
            keys += ("metadata",)
        for key in keys:
            [self.assertIn(key, r.keys()) for r in rules]
        for key in ('deleted', 'deleted_at', 'instance_mappings'):
            [self.assertNotIn(key, r.keys()) for r in rules]

        # verify values
        self.assertEqual(self.access_type, rules[0]["access_type"])
        self.assertEqual(self.access_to, rules[0]["access_to"])
        self.assertEqual('rw', rules[0]["access_level"])
        if utils.is_microversion_ge(version, '2.21'):
            if self.access_type == 'cephx':
                self.assertIsNotNone(rules[0]['access_key'])
            else:
                self.assertIsNone(rules[0]['access_key'])

        # our share id in list and have no duplicates
        gen = [r["id"] for r in rules if r["id"] in rule["id"]]
        msg = "expected id lists %s times in rule list" % (len(gen))
        self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('3bca373e-e54f-49e1-8789-99a383cf4df3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported(RESTRICTED_RULES_VERSION)
    @ddt.data(
        *itertools.product(utils.deduplicate(
            ["2.81", CONF.share.max_api_microversion]), (True, False))
    )
    @ddt.unpack
    def test_list_restricted_rules_from_other_user(
            self, older_version, lock_visibility):
        # create rule
        self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=self.access_type, access_to=self.access_to,
            lock_visibility=lock_visibility, lock_deletion=True)

        rule = (
            self.shares_v2_client.list_access_rules(
                self.share["id"])["access_list"][0])

        rules = self.new_user.shares_v2_client.list_access_rules(
            self.share['id'])['access_list']
        rules_get_lower_version = (
            self.new_user.shares_v2_client.list_access_rules(
                self.share['id'], version=older_version)['access_list'])
        expected_access_to = '******' if lock_visibility else self.access_to
        expected_access_key = (
            '******' if lock_visibility else rule['access_key'])

        # verify values
        rule_latest_rules_api = [r for r in rules if r['id'] == rule['id']][0]
        rule_lower_version_rules_api = [r for r in rules_get_lower_version
                                        if r['id'] == rule['id']][0]
        self.assertEqual(
            expected_access_to, rule_latest_rules_api["access_to"])
        self.assertEqual(
            expected_access_to,
            rule_lower_version_rules_api['access_to'])
        if self.access_type == 'cephx':
            self.assertEqual(expected_access_key,
                             rule_latest_rules_api['access_key'])
            self.assertEqual(
                expected_access_key,
                rule_lower_version_rules_api['access_key'])

    @decorators.idempotent_id('4829265a-eb32-400d-91a0-be06ce31a2ef')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_admin_listing_restricted_rules(self):
        utils.check_skip_if_microversion_not_supported(
            RESTRICTED_RULES_VERSION)

        # create rule
        self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=self.access_type, access_to=self.access_to,
            lock_visibility=True)

        rules = self.admin_shares_v2_client.list_access_rules(
            self.share["id"])['access_list']

        # ensure admin can see rules even if locked
        self.assertEqual(self.access_to, rules[0]["access_to"])
        if self.access_type == 'cephx':
            self.assertIsNotNone(rules[0]['access_key'])
            self.assertFalse(rules[0]['access_key'] == '******')
        else:
            self.assertIsNone(rules[0]['access_key'])

    @decorators.idempotent_id('00202c6c-b4c7-4fa6-933a-562fbffde405')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_admin_delete_restricted_rules(self):
        utils.check_skip_if_microversion_not_supported(
            RESTRICTED_RULES_VERSION)

        # create rule
        rule = self.allow_access(
            self.share["id"], client=self.shares_v2_client,
            access_type=self.access_type, access_to=self.access_to,
            lock_visibility=True, lock_deletion=True, cleanup=False)

        self.admin_shares_v2_client.delete_access_rule(
            self.share['id'], rule['id'], unrestrict=True)

        self.shares_v2_client.wait_for_resource_deletion(
            rule_id=rule['id'], share_id=self.share['id'])

    @decorators.idempotent_id('b77bcbda-9754-48f0-9be6-79341ad1af64')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(*utils.deduplicate(['1.0', '2.9', '2.27', '2.28',
                                 LATEST_MICROVERSION]))
    def test_access_rules_deleted_if_share_deleted(self, version):
        if (utils.is_microversion_lt(version, '2.13') and
                CONF.share.enable_cephx_rules_for_protocols):
            msg = ("API version %s does not support cephx access type, need "
                   "version >= 2.13." % version)
            raise self.skipException(msg)
        if utils.is_microversion_le(version, '2.9'):
            client = self.shares_client
        else:
            client = self.shares_v2_client

        # create share
        share = self.create_share(share_type_id=self.share_type_id)

        # create rule
        rule = self.allow_access(
            share["id"], client=client, access_type=self.access_type,
            access_to=self.access_to, version=version, cleanup=False)

        # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
        if utils.is_microversion_le(version, "2.27"):
            self.assertEqual("new", rule['state'])
        else:
            self.assertEqual("queued_to_apply", rule['state'])

        # delete share
        if utils.is_microversion_eq(version, '1.0'):
            self.shares_client.delete_share(share['id'])
            self.shares_client.wait_for_resource_deletion(share_id=share['id'])
        else:
            self.shares_v2_client.delete_share(share['id'], version=version)
            self.shares_v2_client.wait_for_resource_deletion(
                share_id=share['id'], version=version)

        # verify absence of rules for nonexistent share id
        if utils.is_microversion_eq(version, '1.0'):
            self.assertRaises(lib_exc.NotFound,
                              self.shares_client.list_access_rules,
                              share['id'])
        elif utils.is_microversion_lt(version, '2.45'):
            self.assertRaises(lib_exc.NotFound,
                              self.shares_v2_client.list_access_rules,
                              share['id'], version)
        else:
            self.assertRaises(lib_exc.BadRequest,
                              self.shares_v2_client.list_access_rules,
                              share['id'], version)
